@@ -49,9 +49,9 @@ def make_callability_from_bed(bedfile, window_size):
 
 
 
-def Load_observations_weights_mutrates(obs_file, mutrates_file, weights_file, window_size, haploid):
+def Load_observations_weights_mutrates(obs_file, weights_file, mutrates_file, window_size = 1000, haploid = False):
 
-    obs_counter = defaultdict(lambda: defaultdict(lambda: defaultdict(int)))
+    obs_counter = defaultdict(lambda: defaultdict(lambda: defaultdict(list)))
     haplotypes = defaultdict(int)
 
     with open(obs_file) as data:
@@ -63,23 +63,27 @@ def Load_observations_weights_mutrates(obs_file, mutrates_file, weights_file, wi
                 if haploid:  
                     for i, base in enumerate(genotype):
                         if base != ancestral_base:
-                            obs_counter[chrom][rounded_pos][f'haploid_{i+1}'] += 1
-                            haplotypes[f'haploid_{i+1}'] += 1
+                            obs_counter[chrom][rounded_pos][f'_hap{i+1}'].append(pos)
+                            haplotypes[f'_hap{i+1}'] += 1
                 else:
-                    obs_counter[chrom][rounded_pos]['diploid'] += 1
-                    haplotypes['diploid'] += 1
+                    obs_counter[chrom][rounded_pos][''].append(pos)
+                    haplotypes[''] += 1
 
-    temp_obs = []
+    chroms, starts, variants, obs = [], [], [], []
     for haplotype in sorted(haplotypes):
         for chrom in sorted(obs_counter, key=sortby):
             lastwindow = max(obs_counter[chrom]) + window_size
 
             for window in range(0, lastwindow, window_size):
-                temp_obs.append(obs_counter[chrom][window][haplotype])
+                chroms.append(f'{chrom}{haplotype}')   
+                starts.append(window)
+                variants.append(','.join(obs_counter[chrom][window][haplotype]))  
+                obs.append(len(obs_counter[chrom][window][haplotype])) 
+                
 
     # Read weights file is it exists - else set all weights to 1
     if weights_file is None:
-        weights = np.ones(len(temp_obs)) 
+        weights = np.ones(len(obs)) 
     else:  
         callability = make_callability_from_bed(weights_file, window_size)
         weights = []
@@ -93,7 +97,7 @@ def Load_observations_weights_mutrates(obs_file, mutrates_file, weights_file, wi
 
     # Read mutation rate file is it exists - else set all mutation rates to 1
     if mutrates_file is None:
-        mutrates = np.ones(len(temp_obs)) 
+        mutrates = np.ones(len(obs)) 
     else:  
         callability = make_callability_from_bed(mutrates_file, window_size)
         mutrates = []
@@ -105,52 +109,14 @@ def Load_observations_weights_mutrates(obs_file, mutrates_file, weights_file, wi
                     mutrates.append(callability[chrom][window] / float(window_size))
 
     # Make sure there are no places with obs > 0 and 0 in mutation rate or weight
-    obs = np.zeros(len(temp_obs))
-    for index, (observation, w, m) in enumerate(zip(temp_obs, weights, mutrates)):
+    for index, (observation, w, m) in enumerate(zip(obs, weights, mutrates)):
         if w*m == 0 and observation != 0:
+            print(f'warning, you had {observation} observations but no called bases/no mutation rate at index:{index}. weights:{w}, mutrates:{m}')
             obs[index] = 0
-            print('warning, you had observations but no called bases/no mutation rate')
-            print(index, observation, w, m)
-        else:
-            obs[index] = int(observation)
-
-    return obs.astype(int), np.array(mutrates).astype(float), np.array(weights).astype(float)
+            
 
 
-
-def Get_genome_coordinates(obs_file, window_size, haploid):
-    
-    obs_counter = defaultdict(lambda: defaultdict(lambda: defaultdict(list)))
-    haplotypes = defaultdict(int)
-
-    # Load data
-    with open(obs_file) as data:
-        for line in data:
-            if not line.startswith('chrom'):
-                chrom, pos, ancestral_base, genotype = line.strip().split()
-                rounded_pos = int(pos) - int(pos) % window_size
-
-                if haploid:
-                    for i, base in enumerate(genotype):
-                        if base != ancestral_base:
-                            obs_counter[chrom][rounded_pos][f'_hap{i+1}'].append(pos)
-                            haplotypes[f'_hap{i+1}'] += 1
-                else:
-                    obs_counter[chrom][rounded_pos][''].append(pos)
-                    haplotypes[''] += 1
-
-    # Make list of chromosomes, starts and variants in each window
-    chroms, starts, variants = [], [], []
-    for haplotype in sorted(haplotypes):
-        for chrom in sorted(obs_counter, key=sortby):
-            lastwindow = max(obs_counter[chrom]) + window_size
-
-            for window in range(0, lastwindow, window_size):
-                chroms.append(f'{chrom}{haplotype}')   
-                starts.append(window)
-                variants.append(','.join(obs_counter[chrom][window][haplotype]))
-
-    return chroms, starts, variants
+    return np.array(obs).astype(int), chroms, starts, variants, np.array(mutrates).astype(float), np.array(weights).astype(float)
 
 
 # ----------------------------------------------------------------------------------------------------------------------------------------------------------------
@@ -341,7 +307,7 @@ def clean_files(filename):
 
 
 # Find variants from admixed population
-def find_admixed_variants(variants_list):
+def flatten_list(variants_list):
 
     flattened_list = []
     for bin in variants_list:
@@ -352,7 +318,7 @@ def find_admixed_variants(variants_list):
             else:
                 flattened_list.append(bin)
 
-    return flattened_list
+    return ','.join(flattened_list)
 
 
 def convert_to_bases(genotype, ref, alt):
