@@ -4,11 +4,11 @@ from numba import njit
 import json
 import math
 
-from helper_functions import Load_observations_weights_mutrates, find_runs, Annotate_with_ref_genome, Make_folder_if_not_exists, Get_genome_coordinates, find_admixed_variants
+from helper_functions import find_runs, Annotate_with_ref_genome, Make_folder_if_not_exists, flatten_list
 
 # ----------------------------------------------------------------------------------------------------------------------------------------------------------------
 # HMM Parameter Class
-# ------------------------------------------------------
+# ----------------------------------------------------------------------------------------------------------------------------------------------------------------
 class HMMParam:
     def __init__(self, state_names, starting_probabilities, transitions, emissions): 
         self.state_names = np.array(state_names)
@@ -29,6 +29,10 @@ class HMMParam:
         
 # Read HMM parameters from a json file
 def read_HMM_parameters_from_file(filename):
+
+    if filename is None:
+        return get_default_HMM_parameters()
+
     with open(filename) as json_file:
         data = json.load(json_file)
 
@@ -52,10 +56,6 @@ def write_HMM_to_file(hmmparam, outfile):
         out.write(json_string)
 
 
-
-# ----------------------------------------------------------------------------------------------------------------------------------------------------------------
-# HMM functions
-# ----------------------------------------------------------------------------------------------------------------------------------------------------------------
 def logoutput(hmm_parameters, loglikelihood, iteration):
 
     n_states = len(hmm_parameters.emissions)
@@ -74,6 +74,12 @@ def logoutput(hmm_parameters, loglikelihood, iteration):
     print(iteration, round(loglikelihood, 4), print_starting_probabilities, print_emissions, print_transitions, sep = '\t')
 
 
+
+# ----------------------------------------------------------------------------------------------------------------------------------------------------------------
+# HMM functions
+# ----------------------------------------------------------------------------------------------------------------------------------------------------------------
+
+
 def Emission_probs_poisson(emissions, observations, weights, mutrates):
     n = len(observations)
     n_states = len(emissions)
@@ -104,7 +110,7 @@ def forward(probabilities, transitions, init_start):
 
     for t in range(n):
         if t == 0:
-            forwards_in[t,:]=  init_start  * probabilities[t,:]
+            forwards_in[t,:] = init_start  * probabilities[t,:]
             scale_param[t] = np.sum( forwards_in[t,:])
             forwards_in[t,:] = forwards_in[t,:] / scale_param[t]
         else:
@@ -174,27 +180,14 @@ def TrainBaumWelsch(hmm_parameters, weights, obs, mutrates):
 # ----------------------------------------------------------------------------------------------------------------------------------------------------------------
 # Train
 # ----------------------------------------------------------------------------------------------------------------------------------------------------------------
-def TrainModel(obs_file, weights_file, mutation_rates_file, parameters_file, outfile, window_size, haploid):
-
-    obs, mutrates, weights = Load_observations_weights_mutrates(obs_file, mutation_rates_file, weights_file, window_size, haploid)
-    hmm_parameters = read_HMM_parameters_from_file(filename = parameters_file)
-    epsilon = 1e-3
-
-    # Print parameters to screen
-    print('-' * 40)
-    print(hmm_parameters)
-    print('> number of windows:', len(obs), '. Number of snps = ', sum(obs))
-    print('> total callability:', round(np.sum(weights) / len(obs),2) )
-    print('> average mutation rate per bin:', round(np.sum(mutrates * weights) / np.sum(weights), 2) )
-    print('> Output is',outfile) 
-    print('-' * 40)
+def TrainModel(obs, mutrates, weights, hmm_parameters, epsilon = 1e-3, maxiterations = 1000):
 
     # Get probability of sequece with start parameters
     previous_loglikelihood = GetProbability(hmm_parameters, weights, obs, mutrates)
     logoutput(hmm_parameters, previous_loglikelihood, 0)
     
     # Train parameters using Baum Welch algorithm
-    for i in range(1,1000):
+    for i in range(1,maxiterations):
         hmm_parameters = TrainBaumWelsch(hmm_parameters, weights, obs, mutrates)
         new_loglikelihood = GetProbability(hmm_parameters, weights, obs, mutrates)
         logoutput(hmm_parameters, new_loglikelihood, i)
@@ -204,107 +197,77 @@ def TrainModel(obs_file, weights_file, mutation_rates_file, parameters_file, out
 
         previous_loglikelihood = new_loglikelihood
 
-    # Write the optimal parameters
-    write_HMM_to_file(hmm_parameters, outfile)
+    # Write the optimal parametersf
+    return hmm_parameters
+    
 
 # ----------------------------------------------------------------------------------------------------------------------------------------------------------------
 # Decode
 # ----------------------------------------------------------------------------------------------------------------------------------------------------------------
 
-def DecodeModel(obs_file, weights_file, mutation_rates_file, parameters_file, output, window_size, haploid, decode_per_bin, admixpop):
+def DecodeModel(obs, chroms, starts, variants, mutrates, weights, hmm_parameters):
     
-    obs, mutrates, weights = Load_observations_weights_mutrates(obs_file, mutation_rates_file, weights_file, window_size, haploid)
-    hmm_parameters = read_HMM_parameters_from_file(filename = parameters_file)
-    chroms, starts, variants = Get_genome_coordinates(obs_file, window_size, haploid)
-
-    # Print parameters to screen
-    print('-' * 40)
-    print(hmm_parameters)  
-    print('> number of windows:', len(obs), '. Number of snps = ', sum(obs))
-    print('> total callability:', round(np.sum(weights) / len(obs),2) )
-    print('> average mutation rate per bin:', round(np.sum(mutrates * weights) / np.sum(weights), 2) )
-    print('> Output is',output) 
-    print('-' * 40)
-
     # Posterior decode the file
     emissions = Emission_probs_poisson(hmm_parameters.emissions, obs, weights, mutrates)
     forward_probs, scales = forward(emissions, hmm_parameters.transitions, hmm_parameters.starting_probabilities)
     backward_probs = backward(emissions, hmm_parameters.transitions, scales)
     post_seq = (forward_probs * backward_probs).T
 
+    window_size = starts[2] - starts[1]
+
+    segments = []
+    for (chrom, chrom_start_index, chrom_length_index) in find_runs(chroms):
+        state_with_highest_prob = np.argmax(post_seq[:,chrom_start_index:chrom_start_index + chrom_length_index-1], axis = 0)
+        for (state, start_index, length_index) in find_runs(state_with_highest_prob):
+
+            start_index = start_index + chrom_start_index
+            end_index = start_index + length_index
+
+            genome_start = starts[start_index]
+            genome_end = starts[start_index + length_index - 1]
+            genome_length =  length_index * window_size
+
+            snp_counter = np.sum(obs[start_index:end_index])
+            mean_prob = round(np.mean(post_seq[state, start_index:end_index]), 5)
+            variants_segment = flatten_list(variants[start_index:end_index])
+
+            segments.append([chrom, genome_start,  genome_end, genome_length, hmm_parameters.state_names[state], mean_prob, snp_counter, variants_segment]) 
+        
+    return segments
+
+
+
+def Write_Decoded_output(output, segments, obs_file = None, admixpop_file = None):
+    
     # write summary file 
     Make_folder_if_not_exists(output)
 
-    if not decode_per_bin:
-        with open(output,'w') as out: 
-
-            if admixpop is not None:
-                admix_pop_variants, admixpop_names = Annotate_with_ref_genome(admixpop, obs_file)
-                header = 'chrom\tstart\tend\tlength\tstate\tmean_prob\tsnps\tadmixpopvariants\t{}\n'.format('\t'.join(admixpop_names))
-            else:
-                header = 'chrom\tstart\tend\tlength\tstate\tmean_prob\tsnps\n'
-
-            out.write(header)
-
-            for (chrom, chrom_start_index, chrom_length_index) in find_runs(chroms):
-
-                state_with_highest_prob = np.argmax(post_seq[:,chrom_start_index:chrom_start_index + chrom_length_index-1], axis = 0)
-
-                for (state, start_index, length_index) in find_runs(state_with_highest_prob):
-
-                    start_index = start_index + chrom_start_index
-                    end_index = start_index + length_index
-
-                    genome_start = starts[start_index]
-                    genome_end = starts[start_index + length_index - 1]
-                    genome_length =  length_index * window_size
-
-                    snp_counter = np.sum(obs[start_index:end_index])
-                    mean_prob = np.mean(post_seq[state, start_index:end_index])
-
-
-                    if admixpop is not None:
-                        mylist = find_admixed_variants(variants[start_index:end_index])
-                        archiac_variants_dict = defaultdict(int)
-                        for snp_position in mylist:
-                            variant = admix_pop_variants[f'{chrom}_{snp_position}']
-                            if variant != '':
-                                if '|' in variant:
-                                    for ind in variant.split('|'):
-                                        archiac_variants_dict[ind] += 1
-                                else:
-                                    archiac_variants_dict[variant] += 1
-
-                                archiac_variants_dict['total'] += 1
-
-                        archaic_variants = '\t'.join([str(archiac_variants_dict[x]) for x in ['total'] + admixpop_names])
-                        print(chrom, genome_start, genome_end, genome_length, hmm_parameters.state_names[state], round(mean_prob, 5), snp_counter, archaic_variants, sep = '\t', file = out)
-                    else:
-                        print(chrom, genome_start, genome_end, genome_length, hmm_parameters.state_names[state], round(mean_prob, 5), snp_counter, sep = '\t', file = out)
-
-
-
-    # write posterios prob for each bin (if specified)    
-    if decode_per_bin:
-
-        if admixpop is not None:
-            header = 'chrom\tstart\tobservations\tMostlikely\t{}\tvariants\tadmixpopvariants\n'.format('\t'.join(hmm_parameters.state_names))
-            admix_pop_variants, admixpop_names = Annotate_with_ref_genome(admixpop, obs_file)
+    with open(output,'w') as out: 
+        if admixpop_file is not None:
+            admix_pop_variants, admixpop_names = Annotate_with_ref_genome(admixpop_file, obs_file)
+            out.write('chrom\tstart\tend\tlength\tstate\tmean_prob\tsnps\tadmixpopvariants\t{}\n'.format('\t'.join(admixpop_names)))
         else:
-            header = 'chrom\tstart\tobservations\tMostlikely\t{}\tvariants\n'.format('\t'.join(hmm_parameters.state_names))
+            out.write('chrom\tstart\tend\tlength\tstate\tmean_prob\tsnps\n')
 
-        with open(output,'w') as out:
-            out.write(header)
-            state_with_highest_prob = np.argmax(post_seq, axis = 0)
-            for index, (chrom, variant_in_bin, obs_in_bin) in enumerate(zip(chroms, variants, obs)):
+        for chrom, genome_start, genome_end, genome_length, state, mean_prob, snp_counter, variants in segments:
 
-                state = hmm_parameters.state_names[state_with_highest_prob[index]]
-                probability_of_states = '\t'.join([str(round(val,4)) for val in post_seq[:, index]])
+            if admixpop_file is not None:
+                archiac_variants_dict = defaultdict(int)
+                for snp_position in variants.split(','):
+                    variant = admix_pop_variants[f'{chrom}_{snp_position}']
+                    if variant != '':
+                        if '|' in variant:
+                            for ind in variant.split('|'):
+                                archiac_variants_dict[ind] += 1
+                        else:
+                            archiac_variants_dict[variant] += 1
+
+                        archiac_variants_dict['total'] += 1
+
+                archaic_variants = '\t'.join([str(archiac_variants_dict[x]) for x in ['total'] + admixpop_names])
+                print(chrom, genome_start, genome_end, genome_length, state, mean_prob, snp_counter, archaic_variants, sep = '\t', file = out)
+            else:
+                print(chrom, genome_start, genome_end, genome_length, state, mean_prob, snp_counter, sep = '\t', file = out)
 
-                if admixpop is not None:
-                    admix_pop_variants_window = '|'.join([admix_pop_variants[f'{chrom}_{x}'] for x in variant_in_bin.split(',')])
-                    print(chrom, index * window_size, obs_in_bin, state, probability_of_states, variant_in_bin, admix_pop_variants_window, sep = '\t', file =  out)
-                else:
-                    print(chrom, index * window_size, obs_in_bin, state, probability_of_states, variant_in_bin, sep = '\t', file =  out)
 
 
