@@ -1,16 +1,7 @@
-import argparse
-
 from helper_functions import *
 from hmm_functions import *
 from bcf_vcf import *
-
 import numpy as np
-import json
-from collections import defaultdict
-import os
-import itertools
-import difflib
-import sys 
 from glob import glob
 
 
